<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Emissao;

use Financeiro\Entities\Boletreceber;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco\ValidacaoEmissaoBoletoSantanderService;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco\ValidacaoEmissaoBoletoItauService;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco\ValidacaoEmissaoBoletoCaixaService;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco\ValidacaoEmissaoBoletoBrasilService;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco\ValidacaoEmissaoBoletoBanrisulService;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco\ValidacaoEmissaoBoletoBradescoService;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Exception\ExcecaoBoletoGeralService;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Exception\CodigoCompensacaoInvalidoService;

/**
 * Class ValidacaoEmissaoBoletoService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Emissao
 */
abstract class ValidacaoEmissaoBoletoService
{

    protected $boleto;

    /**
     * @param Boletreceber $boleto
     */
    public function __construct(Boletreceber $boleto)
    {
        $this->boleto = $boleto;
    }

    /**
     * @return mixed
     */
    public abstract function valida();

    /**
     * @param Boletreceber $boleto
     * @return null
     * @throws \Exception
     */
    public static function factory(Boletreceber $boleto)
    {
        try {
            self::validacoesGerais($boleto);
            $codigoCompensacao = $boleto->getReducont()->getAgencia()->getBanco()->getCdbanccnab();
            return self::defineBancoValidacao($codigoCompensacao, $boleto);
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    /**
     * @param $codigoCompensacao
     * @param $boleto
     * @return null
     */
    private static function defineBancoValidacao($codigoCompensacao, $boleto)
    {
        if ($codigoCompensacao === GeraEmissaoBoletoService::CODIGO_COMPENSACAO_SANTANDER)
            return new ValidacaoEmissaoBoletoSantanderService($boleto);

		if ($codigoCompensacao === GeraEmissaoBoletoService::CODIGO_COMPENSACAO_ITAU)
            return new ValidacaoEmissaoBoletoItauService($boleto);

		if ($codigoCompensacao === GeraEmissaoBoletoService::CODIGO_COMPENSACAO_CAIXA)
            return new ValidacaoEmissaoBoletoCaixaService($boleto);

		if ($codigoCompensacao === GeraEmissaoBoletoService::CODIGO_COMPENSACAO_BRASIL)
            return new ValidacaoEmissaoBoletoBrasilService($boleto);

        if ($codigoCompensacao === GeraEmissaoBoletoService::CODIGO_COMPENSACAO_BANRISUL)
            return new ValidacaoEmissaoBoletoBanrisulService($boleto);

        if ($codigoCompensacao === GeraEmissaoBoletoService::CODIGO_COMPENSACAO_BRADESCO) {
            return new ValidacaoEmissaoBoletoBradescoService($boleto);
        }else CodigoCompensacaoInvalidoService::CnabInvalido();

        return null;
    }

    /**
     * @param Boletreceber $boleto
     * @throws ExcecaoBoletoGeralService
     */
    private static function validacoesGerais(Boletreceber $boleto)
    {
        if (!$boleto->getCdreducont())
            ExcecaoBoletoGeralService::contaCorrenteNaoInformada();
        if (!$boleto->getCdcartcobbol())
            ExcecaoBoletoGeralService::carteiraNaoInformada();
        if (!$boleto->getReducont()->getCdcedenteban())
            ExcecaoBoletoGeralService::codigoCedenteNaoInformado();
            
        $endeclie = \Geral\Entities\Endeclie::findOneByNrorgAndCdclienteAndIdtpendeclie($boleto->getNrorg(), $boleto->getCdcliente(), 'C');
        
        if (!$endeclie)
            ExcecaoBoletoGeralService::enderecoCobrancaNaoCadastrado();
    }


}
