<?php
namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\Model\Financeiro\Entities\Movbancario;
use Teknisa\Model\Financeiro\Entities\Ittransfbanc;
use Teknisa\Model\Financeiro\Entities\Transfbanc;
use Teknisa\Model\Contabilidade\Entities\Histintegra;
use Teknisa\Model\Geral\Entities\Filial;
use Teknisa\Model\Contabilidade\Entities\Lancccusto;
use Teknisa\Model\Contabilidade\Entities\Lanccont;
use Teknisa\Model\Financeiro\Entities\Movreal;
use Teknisa\Libs\Util\DateUtil;

class Bco11100TransferenciaContaCorrenteFactory
{

    private $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function getIdContabilizacao($nrorg){
        try{
            $sql = <<<SQL
            SELECT IDCTBTRANS
              FROM GERAL_BCO
             WHERE NRORG = :P_NRORG
SQL;
            $params = array(
                'P_NRORG' => $nrorg
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);

            return $std->fetchColumn();
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function validaPeriodoContabil($nrorg, $cdempresa, $dtmovbanc){
        try{
            $sql = <<<SQL
            SELECT 1
              FROM CONTOPECTBL
             WHERE NRORG   = :P_NRORG
               AND CDEMPRESA = :P_CDEMPRESA
               AND :P_DTMOVBANC BETWEEN DTINICCTBL AND DTFINACCTBL
SQL;
            $params = array(
                'P_NRORG' => $nrorg,
                'P_CDEMPRESA' => $cdempresa,
                'P_DTMOVBANC' => $dtmovbanc
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);

            return $std->fetchColumn();
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function validaLancamentoContabil($nrorg, $cdempresa, $nrlanccont){
        try{
            $sql = <<<SQL
            SELECT LC.CDEMPRESA,
                   E.NMEMPRESA,
                   LC.NRLANCCONT,
                   LC.NRSEQLANC,
                   LC.CDCONTCTBL,
                   LC.CDHISTPADR,
                   LC.DTLANCCONT,
                   LC.CDDOCUCONT,
                   LC.IDTIPOLANC,
                   LC.DSHISTLANC,
                   LC.VRLANCAMENT,
                   LC.IDLANCCOME,
                   LC.CDCONTCOME,
                   LC.IDLANCSALDO,
                   LC.IDLANCINTE,
                   LC.IDLANCMANI,
                   LC.CDOPERINS,
                   LC.DTULTATULAN,
                   CC.NMCONTCTBL,
                   CC.CDCONTREDU,
                   CC.IDCONTCCUS
              FROM LANCCONT LC
              JOIN CONTCTBL CC
                ON LC.CDEMPRESA   = :P_CDEMPRESA
               AND LC.NRLANCCONT = :P_NRLANCCONT
               AND LC.NRORG      = :P_NRORG
               AND LC.CDCONTCTBL = CC.CDCONTCTBL
               AND LC.NRORG      = CC.NRORG
              JOIN EMPRESA E
                ON LC.CDEMPRESA = E.CDEMPRESA
               AND LC.NRORG    = E.NRORG
SQL;
            $params = array(
                'P_NRORG' => $nrorg,
                'P_CDEMPRESA' => $cdempresa,
                'P_NRLANCCONT' => $nrlanccont
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);

            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function validaLancCCusto($nrorg, $cdempresa, $nrlanccont){
        try{
            $sql = <<<SQL
            SELECT LC.CDEMPRESA,
                   LC.NRLANCCONT,
                   LC.NRSEQLANC,
                   LC.CDFILIAL,
                   LC.CDCENTCUST,
                   LC.VRLANCCUST,
                   CC.NMCENTCUST,
                   FI.NMFILIAL
              FROM LANCCCUSTO LC
              JOIN FILIAL FI
                ON LC.CDEMPRESA = :P_CDEMPRESA
               AND LC.NRLANCCONT  = :P_NRLANCCONT
               AND LC.NRORG  = :P_NRORG
               AND LC.CDFILIAL    = FI.CDFILIAL
               AND LC.NRORG  = FI.NRORG
              JOIN CENTCUST CC
                ON CC.CDCENTCUST  = LC.CDCENTCUST
               AND CC.NRORG  = LC.NRORG
SQL;
            $params = array(
                'P_NRORG' => $nrorg,
                'P_CDEMPRESA' => $cdempresa,
                'P_NRLANCCONT' => $nrlanccont
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);

            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function validaClassificacaoFinanceira($cdreducont, $cdempresa, $nrorg){
        try{
            $sql = <<<SQL
            SELECT C.IDTIPOCCORR
              FROM CONTCORR C,
                   BANCO B
             WHERE (CDREDUCONT = :P_CDREDUCONT)
               AND (CDEMPRESA  = :P_CDEMPRESA)
               AND (C.CDBANCO  = B.CDBANCO)
               AND (C.NRORG  = B.NRORG)
               AND (C.NRORG  = :P_NRORG)
SQL;
            $params = array(
                'P_NRORG' => $nrorg,
                'P_CDEMPRESA' => $cdempresa,
                'P_CDREDUCONT' => $cdreducont
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);

            return $std->fetchColumn();
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getClassificacaoFinanceira($nrorg){
        try{
            $sql = <<<SQL
            SELECT CDCLASSFINA
              FROM GERAL_FCX
             WHERE NRORG = :P_NRORG
SQL;
            $params = array(
                'P_NRORG' => $nrorg
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);

            return $std->fetchColumn();
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function validaAcessoFilialRelConta($cdoperador, $cdfilial, $nrorg){
        try{
            $sql = <<<SQL
            SELECT DISTINCT FIL.CDFILIAL
              FROM FILIOPER FIL, FILIAL F
             WHERE (FIL.CDOPERADOR = :P_CDOPERADOR)
               AND (FIL.CDFILIAL   = :P_CDFILIAL)
               AND (FIL.NRORG      = :P_NRORG)
               AND (FIL.NRORG      = F.NRORG)
             ORDER BY FIL.CDFILIAL
SQL;
            $params = array(
                'P_NRORG' => $nrorg,
                'P_CDFILIAL' => $cdfilial,
                'P_CDOPERADOR' => $cdoperador
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);
            return $std->fetchColumn();
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function validaAcessoFilialEmpresa($cdoperador, $empresa, $nrorg){
        try{
            $sql = <<<SQL
            SELECT DISTINCT FIL.CDFILIAL
              FROM FILIOPER FIL, FILIAL F
             WHERE (FIL.CDOPERADOR = :P_CDOPERADOR)
               AND (FIL.CDFILIAL   = F.CDFILIAL AND F.CDEMPRESA     = :P_CDEMPRESA)
               AND (F.NRORG        = :P_NRORG)
               AND (F.NRORG        = FIL.NRORG)
             ORDER BY FIL.CDFILIAL
SQL;
            $params = array(
                'P_NRORG' => $nrorg,
                'P_CDEMPRESA' => $empresa,
                'P_CDOPERADOR' => $cdoperador
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);
            return $std->fetchColumn();
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function validaAcessoFilialPorOperador($cdempresa, $nrorg){
        try{
            $sql = <<<SQL
            SELECT CDFILIAL
              FROM FILIAL
             WHERE CDEMPRESA = :P_CDEMPRESA
               AND IDATIVO   = 'S'
               AND NRORG     = :P_NRORG
             ORDER BY CDFILIAL
SQL;
            $params = array(
                'P_NRORG' => $nrorg,
                'P_CDEMPRESA' => $cdempresa
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);
            return $std->fetchColumn();
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getGeralBco($nrorg){
        try{
            $sql = <<<SQL
            SELECT IDPERTRANSCCOR
              FROM GERAL_BCO
             WHERE NRORG = :P_NRORG
SQL;
            $params = array(
                'P_NRORG' => $nrorg
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);
            return $std->fetchColumn();
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getContaCorrente($empresa, $nrorg, $cdreducont)
    {
        $sql = <<<SQL
            SELECT C.CDREDUCONT,
                   C.CDBANCO,
                   C.CDAGENCIA,
                   C.IDTIPOCCORR,
                   C.CDCONTCORR,
                   C.DSCONTCORR,
                   C.CDCONTCTBL,
                   C.CDFILIAL,
                   B.CDPICTCONT
              FROM CONTCORR C
              JOIN BANCO B
                ON (C.CDBANCO  = B.CDBANCO)
               AND ((C.NRORG  = B.NRORG) OR (B.NRORG = 0))
               AND (INSTR(:P_CDREDUCONT ,C.CDREDUCONT) <> 0 )
               AND (CDEMPRESA  = :P_CDEMPRESA)
               AND (C.NRORG = :P_NRORG)
SQL;
        $params = array(
            'P_NRORG' => $nrorg,
            'P_CDEMPRESA' => $empresa,
            'P_CDREDUCONT' => $cdreducont
        );
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getFilialEmpresa($cdoperador, $empresa, $nrorg, $cdfilial){
        try{
            $sql = <<<SQL
            SELECT DISTINCT FIL.CDFILIAL
              FROM FILIOPER FIL, FILIAL F
             WHERE (FIL.CDOPERADOR = :P_CDOPERADOR)
               AND ((FIL.CDFILIAL  = :P_CDFILIAL) OR
                   (FIL.CDFILIAL  = F.CDFILIAL AND F.CDEMPRESA = :P_CDEMPRESA AND :P_CDFILIAL = 'T'))
               AND (F.NRORG        = :P_NRORG)
               AND (F.NRORG        = FIL.NRORG)
             ORDER BY FIL.CDFILIAL
SQL;
            $params = array(
                'P_NRORG' => $nrorg,
                'P_CDEMPRESA' => $empresa,
                'P_CDOPERADOR' => $cdoperador,
                'P_CDFILIAL' => $cdfilial
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getItensContaLongoPrazo($cdempresa, $nrorg, $nrtranbanc){
        $sql = <<<SQL
           SELECT T.CDCLASSFINA,
                  T.CDFILIAL,
                  T.DTMOVFLUXO,
                  T.IDMOVFLUXO,
                  I.IDMOVITBANC,
                  I.VRMOVITBANC,
                  I.CDREDUCONT,
                  I.NRMOVBANC,
                  C.IDTIPOCCORR
             FROM TRANSFBANC T,
                  ITTRANSFBANC I,
                  CONTCORR C
            WHERE (T.CDEMPRESA = :CDEMPRESA)
              AND (T.NRTRANBANC  = :NRTRANBANC)
              AND (T.NRORG       = :NRORG)
              AND (I.CDEMPRESA   = T.CDEMPRESA)
              AND (I.NRTRANBANC  = T.NRTRANBANC)
              AND (I.NRORG       = T.NRORG)
              AND (C.CDREDUCONT  = I.CDREDUCONT)
              and (C.NRORG       = I.NRORG)
              AND (C.IDTIPOCCORR = 'L')
SQL;
        $params = array(
            'CDEMPRESA' => $cdempresa,
            'NRTRANBANC' => $nrtranbanc,
            'NRORG' => $nrorg
        );
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function insereMovbancario($nrorg, $dttranbanc, $vrtranbanc, $dshisttbanc,
                                            $cdreducont, $nrmovbanc, $idmovi)
    {
        try {
            $movbancario = new Movbancario();
            $movbancario->setCdreducont($cdreducont);
            $movbancario->setNrmovbanc($nrmovbanc);
            $movbancario->setDtmovbanc($dttranbanc);
            $movbancario->setIdmovbanc($idmovi);
            $movbancario->setVrmovbanc($vrtranbanc);
            $movbancario->setNrorg($nrorg);
            $movbancario->setDshismbanc($dshisttbanc);
            $movbancario->setIdorigmbanc('T');
            $movbancario->setIdstconext('0');
            $movbancario->setDtinclusao(DateUtil::getDataAtual());
            $movbancario->setIdstatusctbbanc('P');
            return $movbancario;
        }
        catch(\Exception $e){
            throw $e;
        }
    }

    public function insereTransferencia($transferencia, $nrorg, $nrtranbanc)
    {
        $transfbanc = new Transfbanc();
        $transfbanc->setNrorg($nrorg);
        $transfbanc->setNrtranbanc($nrtranbanc);
        $transfbanc->setCdempresa($transferencia['CDEMPRESA']);
        $transfbanc->setCdclassfina($transferencia['CDCLASSFINA']);
        $transfbanc->setCdfilial($transferencia['CDFILIAL']);
        $transfbanc->setDtmovfluxo($transferencia['DTMOVFLUXO']);
        $transfbanc->setIdmovfluxo($transferencia['IDMOVFLUXO']);
        $transfbanc->setDttranbanc($transferencia['DTTRANBANC']);
        $transfbanc->setDshisttbanc($transferencia['DSHISTTBANC']);
        $transfbanc->setVrtranbanc($transferencia['VRMOVITBANC']);
        $transfbanc->setNrlctbtran($transferencia['NRLCTBTRAN']);
        $transfbanc->setDtinclusao(DateUtil::getDataAtual());
        $transfbanc->setIdstatusctbl('P');
        return $transfbanc;
    }



    public function insereItemTransferencia($nrorg, $cdempresa, $vrtranbanc, $cdreducont,
                                                   $nrtranbanc, $nrmovbanc, $idmovi)
    {
        try {
            $ittransfbanc = new Ittransfbanc();
            $ittransfbanc->setNrorg($nrorg);
            $ittransfbanc->setCdempresa($cdempresa);
            $ittransfbanc->setVrmovitbanc($vrtranbanc);
            $ittransfbanc->setCdreducont($cdreducont);
            $ittransfbanc->setNrtranbanc($nrtranbanc);
            $ittransfbanc->setIdmovitbanc($idmovi);
            $ittransfbanc->setDtinclusao(DateUtil::getDataAtual());
            $ittransfbanc->setNrmovbanc($nrmovbanc);
            return $ittransfbanc;
        }
        catch(\Exception $e){
            throw $e;
        }
    }

    public function insereSaldoContaCorrente($nrorg, $cdempresa, $vrtranbanc, $cdreducont,
                                                   $nrtranbanc, $nrmovbanc, $idmovi)
    {
        try {
            $ittransfbanc = new Ittransfbanc();
            $ittransfbanc->setNrorg($nrorg);
            $ittransfbanc->setCdempresa($cdempresa);
            $ittransfbanc->setVrmovitbanc($vrtranbanc);
            $ittransfbanc->setCdreducont($cdreducont);
            $ittransfbanc->setNrtranbanc($nrtranbanc);
            $ittransfbanc->setIdmovitbanc($idmovi);
            $ittransfbanc->setDtinclusao(DateUtil::getDataAtual());
            $ittransfbanc->setNrmovbanc($nrmovbanc);
            return $ittransfbanc;
        }
        catch(\Exception $e){
            throw $e;
        }
    }

    public function getMovreal($nrorg, $nrseqmovreba, $cdsequexba, $cdreducont){
        try {
            $params = array();
            $params['nrorg'] = $nrorg;
            $params['nrseqmovreba'] = $nrseqmovreba;
            $params['cdsequexba'] = $cdsequexba;
            $params['cdreducont'] = $cdreducont;
            $movreal = $this->entityManager->getRepository(Movreal::class)->findOneBy($params);
            return $movreal;
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function atualizaMovreal(Movreal $movreal){
        try {
            $movreal->setIdstconmvban(0);
            $movreal->setIdconcreal(null);
            $movreal->setNrmovbare(null);
            return $movreal;
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function getMovbancario($nrmovbanc, $cdreducont, $nrorg){
        try {
            $params = array();
            $params['nrorg'] = $nrorg;
            $params['nrmovbanc'] = $nrmovbanc;
            $params['cdreducont'] = $cdreducont;
            $movbancario = $this->entityManager->getRepository(Movbancario::class)->findOneBy($params);
            return $movbancario;
        }catch(\Exception $e){
            throw $e;
        }
    }

    /*public function getIttransfbanc($empresa, $nrtranbanc, $cdreducont, $nrorg){
        try {
            $params = array();
            $params['nrorg'] = $nrorg;
            $params['cdempresa'] = $empresa;
            $params['cdreducont'] = $cdreducont;
            $params['nrtranbanc'] = $nrtranbanc;
            $ittransfbanc = $this->entityManager->getRepository(Ittransfbanc::class)->findOneBy($params);
            return $ittransfbanc;
        }catch(\Exception $e){
            throw $e;
        }
    }*/

    public function getIttransfbanc($empresa, $nrtranbanc, $nrorg){
        try {
            $params = array();
            $params['nrorg'] = $nrorg;
            $params['cdempresa'] = $empresa;
            $params['nrtranbanc'] = $nrtranbanc;
            $ittransfbanc = $this->entityManager->getRepository(Ittransfbanc::class)->findBy($params);
            return $ittransfbanc;
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function getTransfbanc($empresa, $nrtranbanc, $nrorg){
        try {
            $params = array();
            $params['nrorg'] = $nrorg;
            $params['cdempresa'] = $empresa;
            $params['nrtranbanc'] = $nrtranbanc;
            $transfbanc = $this->entityManager->getRepository(Transfbanc::class)->findOneBy($params);
            return $transfbanc;
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function getHistoricoPadrao($nrorg, $cdoperacao){
        $params = array();
        $params['cdoperacao'] = $cdoperacao;
        $params['nrorg'] = $nrorg;
        $histintegra = $this->entityManager->getRepository(Histintegra::class)->findOneBy($params);
        $historico = ($histintegra)? $histintegra->getCdhistpadr() : false;
        return $historico;
    }

    public function getFilial($nrorg, $cdempresa){
        $params = array();
        $params['cdempresa'] = $cdempresa;
        $params['nrorg'] = $nrorg;
        $filial = $this->entityManager->getRepository(Filial::class)->findBy($params);
        return $filial;
    }

    public function updateTransferencia($nrorg, $cdempresa, $nrtranbanc, $dshisttbanc, $vrmovitbanc)
    {
        try {
            $params = array();
            $params['nrorg'] = $nrorg;
            $params['cdempresa'] = $cdempresa;
            $params['nrtranbanc'] = $nrtranbanc;
            $transfbanc = $this->entityManager->getRepository(Transfbanc::class)->findOneBy($params);
            $transfbanc->setDshisttbanc($dshisttbanc);
            $transfbanc->setVrtranbanc($vrmovitbanc);
            return $transfbanc;
        }
        catch(\Exception $e){
            throw $e;
        }
    }

    public function updateItemTransferencia($nrorg, $cdempresa, $nrtranbanc, $cdreducont, $vrmovitbanc)
    {
        try {
            $params = array();
            $params['nrorg'] = $nrorg;
            $params['cdempresa'] = $cdempresa;
            $params['nrtranbanc'] = $nrtranbanc;
            $params['cdreducont'] = $cdreducont;
            $ittransfbanc = $this->entityManager->getRepository(Ittransfbanc::class)->findOneBy($params);
            $ittransfbanc->setVrmovitbanc($vrmovitbanc);
            return $ittransfbanc;
        }
        catch(\Exception $e){
            throw $e;
        }
    }

    public function updateMovbancario($nrorg, $dshisttbanc, $cdreducont, $nrmovbanc, $vrmovitbanc)
    {
        try {
            $params = array();
            $params['cdreducont'] = $cdreducont;
            $params['nrmovbanc'] = $nrmovbanc;
            $params['nrorg'] = $nrorg;
            $movbancario = $this->entityManager->getRepository(Movbancario::class)->findOneBy($params);
            $movbancario->setDshismbanc($dshisttbanc);
            $movbancario->setVrmovbanc($vrmovitbanc);
            return $movbancario;
        }
        catch(\Exception $e){
            throw $e;
        }
    }

    public function getLancCCusto($nrorg, $cdempresa, $nrlanccont){
        $params = array();
        $params['cdempresa'] = $cdempresa;
        $params['nrorg'] = $nrorg;
        $params['nrlanccont'] = $nrlanccont;
        $lancCCusto = $this->entityManager->getRepository(Lancccusto::class)->findBy($params);
        return $lancCCusto;
    }

    public function getLancamentoContabil($nrorg, $cdempresa, $nrlanccont){
        $params = array();
        $params['cdempresa'] = $cdempresa;
        $params['nrorg'] = $nrorg;
        $params['nrlanccont'] = $nrlanccont;
        $lanccont = $this->entityManager->getRepository(Lanccont::class)->findBy($params);
        return $lanccont;
    }
}