<?php
namespace Teknisa\FIN\Services;

include('ImportacaoBancaria/Connection.php');

use Teknisa\FIN\Factories\ImportacaoBancariaFactory;
use Teknisa\FIN\Modules\ContaPagar\Services\RetornoPagarService;
use Teknisa\FIN\Modules\ContaPagar\Services\Pag30000RetornoDDAService;
use Teknisa\FIN\Modules\ContaReceber\Services\RetornoCobrancaService;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Services\Bco54200ExtratoBancario;
use Teknisa\Libs\Util\NewCode;

/**
 * Class ImportacaoBancariaService
 * @package Teknisa\FIN\Service
 */
class ImportacaoBancariaService { 
    
    const ATIVO       = 'S';
    
    const PENDENTE    = 'P';
    const ERRO        = 'E';
    const CONCLUIDA   = 'C';
    
    const PAGAMENTO   = 'P';
    const RECEBIMENTO = 'R';
    const OFX         = 'O';
    const DDA         = 'D';
    const EXTRATO     = 'E';
    
    private static $conn;
    private static $nrorg;
    private static $cdempresa;
    private static $cdfilial;
    private static $cdusuario;
    private static $cdsenha;
    private static $cdoperador;
    private static $header;
    
    protected $entityManager;
    protected $importacaoBancariaFactory;
    protected $bco54200ExtratoBancario;
    protected $retornoPagarService;
    protected $retornoCobrancaService;
    protected $pag30000RetornoDDAService;
    protected $newCode;
    
    public function __construct($entityManager, ImportacaoBancariaFactory $importacaoBancariaFactory, Bco54200ExtratoBancario $bco54200ExtratoBancario, RetornoPagarService $retornoPagarService, RetornoCobrancaService $retornoCobrancaService, Pag30000RetornoDDAService $pag30000RetornoDDAService, NewCode $newCode){
        $this->entityManager             = $entityManager;
    	$this->importacaoBancariaFactory = $importacaoBancariaFactory;
    	$this->bco54200ExtratoBancario   = $bco54200ExtratoBancario;
        $this->retornoPagarService       = $retornoPagarService;
        $this->retornoCobrancaService    = $retornoCobrancaService;
        $this->pag30000RetornoDDAService = $pag30000RetornoDDAService;
        $this->newCode                   = $newCode;
    }
    
    /**
     * Method: importaDadosBancarios()
     * 
     * Description: Essa função tem como finalidade importar os dados bancários diponibilizados
     * no WebService disponibilizado pela Accesstage para integração com o ERP
     */
    public function importaDadosBancarios(){
        try {
            // Busca Parametrização Faixa de Controle
            $paramtrckidacs = $this->importacaoBancariaFactory->getParamTrckIdAcs();
            
            if($paramtrckidacs){
                foreach($paramtrckidacs as $param){
                    // Parâmetros Iniciais
                    self::$nrorg      = $param['NRORG'];
                    self::$cdempresa  = $param['CDEMPRESA'];
                    self::$cdfilial   = $param['CDFILIAL'];
                    self::$cdusuario  = $param['CDUSUARIO'];
                    self::$cdsenha    = $param['CDSENHA'];
                    self::$cdoperador = $param['CDOPERADOR'];
                    
                    // Conecta Web Service
                    $this->conectaWebService();
                    
                    // Busca Faixa de Controle
                    $msgsDisponiveis = $this->buscaTrackId();
                    
                    if($msgsDisponiveis){
                        // Insere Faixa de Controle - Web Service
                        $this->insereTrackId($msgsDisponiveis);
                        
                        // Importa Faixa de Controle
                        $this->importaTrackId($msgsDisponiveis);
                        
                        // Confirma Faixa de Controle
                        //$this->confirmaTrackId($msgsDisponiveis);
                    }else var_dump('Não há mensagens a serem importadas para o usuário: '.self::$cdusuario.'.');
                }
            }else throw new \Exception('Operação bloqueada. Parâmetros para importação bancária não configurados.');
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Method: importaDadosBancarios()
     * 
     * Description: Essa função tem como finalidade importar os dados bancários diponibilizados
     * pelo Client
     */
    public function importaDadosBancariosClient(){
        try {
            // Busca Parametrização Faixa de Controle
            $paramtrckidacs = $this->importacaoBancariaFactory->getParamTrckIdAcs();
            
            if($paramtrckidacs){
                foreach($paramtrckidacs as $param){
                    // Parâmetros Iniciais
                    self::$nrorg      = $param['NRORG'];
                    self::$cdempresa  = $param['CDEMPRESA'];
                    self::$cdfilial   = $param['CDFILIAL'];
                    self::$cdusuario  = $param['CDUSUARIO'];
                    self::$cdsenha    = $param['CDSENHA'];
                    self::$cdoperador = $param['CDOPERADOR'];
            
                    // Busca Faixa de Controle - Client
                    $msgsDisponiveis = $this->buscaTrackIdClient();
                    
                    if($msgsDisponiveis){
                        // Insere Faixa de Controle
                        $this->insereTrackId($msgsDisponiveis);
                        
                        // Importa Faixa de Controle
                        $this->importaTrackId($msgsDisponiveis);
                    }else var_dump('Não há mensagens a serem importadas para o usuário: '.self::$cdusuario.'.');
                }
            }else throw new \Exception('Operação bloqueada. Parâmetros para importação bancária não configurados.');
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Method: conectaWebService()
     *
     * Description: Gera conexão via SoapClient através da URL disponibilizada pela AccesStage.
     * A conexão fica preparada para ser possível acessar o procedimento "process".
     * Para mais informações sobre serviços soap e a documentação das
     * classes SoapClient e SoapServer e seus respectivos metódos,
     * consulte - http://php.net/manual/en/book.soap.php.
     */
    private function conectaWebService(){
        try {
            // Definição de Fuso Horário
            date_default_timezone_set("America/Sao_Paulo");
            
            //Configurações para protocolo de segurança que é lido no header do Soap para todas as requisições
            $wsseNamespace = 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd';
            $security = new \SoapVar(
                array(new \SoapVar(
                    array(
                        new \SoapVar(self::$cdusuario, XSD_STRING, null, null, 'Username', $wsseNamespace),
                        new \SoapVar(self::$cdsenha, XSD_STRING, null, null, 'Password', $wsseNamespace)
                    ), 
                    SOAP_ENC_OBJECT, 
                    null, 
                    null, 
                    'UsernameToken', 
                    $wsseNamespace
                )), 
                SOAP_ENC_OBJECT
            );
    
            self::$header = new \SoapHeader($wsseNamespace, 'Security', $security, false);
        }catch(\Exception $e){
            echo($e->getMessage());
        }
    }

    /**
     * Method: buscaTrackId()
     *
     * Description: Função utilizada para obter a Tracking Id de todos os documentos do tipo esperado.
     * Retornará um array contendo, em cada posição, um número de TrackingId.
     */
    private function buscaTrackId(){
       try{
            $listamsgdisponivel = new \SoapClient(LISTAMSGDISPONIVEL, array("keep_alive" => true));
            
            $listamsgdisponivel->__setSoapHeaders(self::$header);
            if (!$listamsgdisponivel)
                throw new \Exception("Operação abortada. Não foi possível obter conexão para listar as mensagens disponíveis.");
            
            $response = $listamsgdisponivel->__soapCall('process', array());
            if(!$response)
                throw new \Exception("Operação abortada. Não foi possível listar as mensagens disponíveis.");
            
            $msgDisponiveis = isset($response->MsgDisponivel)?$response->MsgDisponivel:null;
            
            if($msgDisponiveis){
                foreach($msgDisponiveis as $msgDisponivel){
                    // Recupera Conteúdo da Faixa de Controle
                    $msgDisponivel->dsconttrckidacs = $this->recuperaMsgDisponivel($msgDisponivel);
                }
            }
            
            return $msgDisponiveis;
        }catch(\Exception $e){
            echo($e->getMessage());
        }
    }

    /**
     * Method: buscaTrackIdClient()
     *
     * Description: Função utilizada para obter a Tracking Id de todos os documentos do tipo esperado.
     * Retornará um array contendo, em cada posição, um número de TrackingId.
     */
    private function buscaTrackIdClient(){
       try{
            $msgsDisponiveis = array();
            
            // Diretorio Client
            $dir     = '/tmp/financeiro/'.self::$cdusuario.'/';
            
            foreach (glob("$dir*.{txt,TXT,ret,RET,ofx,OFX}", GLOB_BRACE) as $file) {
                $handle = fopen ($file, "r");
                $conteudo = fread ($handle, filesize ($file));
                fclose ($handle);
                
                $msgsDisponivel = new \stdClass();
                $msgsDisponivel->trkIdIn          = basename($file);
                $msgsDisponivel->tipoDocumento    = basename($file);
                $msgsDisponivel->nmeEmpresaOrigem = "";
                $msgsDisponivel->dtaHoraDisp      = "";
                $msgsDisponivel->dsconttrckidacs  = $conteudo;
            
                array_push($msgsDisponiveis, $msgsDisponivel);
            }
            
            return $msgsDisponiveis;
        }catch(\Exception $e){
            echo($e->getMessage());
        }
    }
    
    /**
     * Method: insereTrackId()
     *
     * Description: Método para inserção da Faixa de Controle no Banco de Dados
     *
     * @param $msgsDisponiveis
     */
    private function insereTrackId($msgsDisponiveis){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            foreach ($msgsDisponiveis as $msgDisponivel) {
                // Busca Faixa de Controle
                $ctrltrckidacs = $this->importacaoBancariaFactory->getCtrltrckidacs($msgDisponivel->trkIdIn);
                if(!$ctrltrckidacs){
                    // Gera Novo Código
                    $msgDisponivel->nrctrltrckidacs = $this->newCode->getCodeByTableName('CTRLTRCKIDACS', self::$nrorg);
                    
                    // Formata Data
                    $msgDisponivel->dtaHoraDisp = $this->formatDate($msgDisponivel->dtaHoraDisp);
                    
                    // Verifica Tipo de Arquivo
                    $msgDisponivel->idtparquivo = $this->verificaTipoArquivo($msgDisponivel);
                    
                    // Insere Faixa de Controle
                    $this->importacaoBancariaFactory->insertCtrltrckidacs(
                        $msgDisponivel->nrctrltrckidacs, $msgDisponivel->trkIdIn, $msgDisponivel->tipoDocumento,
                        $msgDisponivel->nmeEmpresaOrigem, $msgDisponivel->dtaHoraDisp, $msgDisponivel->idtparquivo, self::$nrorg, self::$cdempresa,
                        self::$cdfilial, self::$cdoperador
                    );
                    
                    // Insere Conteudo da Faixa de Controle
                    $this->importacaoBancariaFactory->insertConttrckidacs(
                        $msgDisponivel->nrctrltrckidacs, $msgDisponivel->trkIdIn, $msgDisponivel->dsconttrckidacs, self::$nrorg
                    );
                }else $msgDisponivel->nrctrltrckidacs = $ctrltrckidacs['NRCTRLTRCKIDACS'];
            }
            
            $this->entityManager->flush();
            $connection->commit();
        }catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Method: insereConteudoTrackId()
     *
     * Description: Insere Conteúdo da Faixa de Controle
     *
     * @param $msgDisponivel
     */
    private function insereConteudoTrackId($msgsDisponiveis){
        try{
            foreach($msgsDisponiveis as $msgDisponivel){
                // Busca Conteúdo Faixa de Controle
                $conttrckidacs = $this->importacaoBancariaFactory->getConttrckidacs($msgDisponivel->nrctrltrckidacs, $msgDisponivel->trkIdIn);
                if(!$conttrckidacs){
                    // Insere Conteudo da Faixa de Controle
                    $this->importacaoBancariaFactory->insertConttrckidacs(
                        $msgDisponivel->nrctrltrckidacs, $msgDisponivel->trkIdIn, $msgDisponivel->dsconttrckidacs, self::$nrorg
                    );
                }
            }
        }catch(\Exception $e){
            throw $e;
        }
    }
     
    /**
     * Method: recuperaMsgDisponivel()
     *
     * Description: Recupera os conteúdos da mensagens diponibilizadas
     * através do WebService
     *
     * @param $msgDisponivel
     */
    private function recuperaMsgDisponivel($msgDisponivel){
       try{
            $recuperamsg = new \SoapClient(RECUPERAMSG, array("keep_alive" => true));
                
            $recuperamsg->__setSoapHeaders(self::$header);
            if(!$recuperamsg)
                throw new \Exception("Operação abortada. Não foi possível obter conexão para recuperar as mensagens.");
            
            $response = $recuperamsg->__soapCall('process', array([ 'trackingId' => $msgDisponivel->trkIdIn ]));
            if(!$response)
                throw new \Exception("Operação abortada. Não foi possível recuperar as mensagens.");
            
    		$conteudoMensagem = $response->dscConteudoMensagem;
    		$response->dscConteudoMensagem = $response->flgCompactacao?base64_decode($conteudoMensagem):$conteudoMensagem;

            return $response->dscConteudoMensagem;
        }catch(\Exception $e){
            echo($e->getMessage());
        }
    }
    
    /**
     * Method: verificaTipoArquivo()
     *
     * Description: Verifica Tipo de Arquivo da Faixa de Controle do Banco de Dados.
     *
     * @param $msgDisponivel
     */
    private function verificaTipoArquivo($msgDisponivel){
        // Verifica Tipo de Arquivo da Faixa de Controle
        if ((substr($msgDisponivel->dsconttrckidacs, 0, 9) == 'OFXHEADER') !== false)
            $idtparquivo = self::OFX;
        elseif (boolval(preg_match('/341|033/i', substr($msgDisponivel->dsconttrckidacs, 0, 3))) !== false)
            if(((substr($msgDisponivel->dsconttrckidacs, 495, 1)) == 'G') && ((substr($msgDisponivel->dsconttrckidacs, 736, 1)) == 'H'))
                 $idtparquivo = self::DDA;
            elseif(((substr($msgDisponivel->dsconttrckidacs, 497, 1)) == 'T') && ((substr($msgDisponivel->dsconttrckidacs, 739, 1)) == 'U'))
                 $idtparquivo = self::RECEBIMENTO;
            else $idtparquivo = self::PAGAMENTO;
        else $idtparquivo = self::EXTRATO;
        
        return $idtparquivo;
    }
    
    /*
     * Method: importaTrackId()
     *
     * Description: Importa o arquivo recebidos através do WebService separando
     * as importações por cada tipo.
     */
    private function importaTrackId() {
        // Busca Faixas de Controle Pendentes de Importação
        $msgsDisponiveis = $this->importacaoBancariaFactory->getConttrckidacsByIdstatimport(self::PENDENTE);
        
        if($msgsDisponiveis){
            foreach ($msgsDisponiveis as $msgDisponivel) {
                // Verifica Tipo de Arquivo da Faixa de Controle
                switch ($msgDisponivel['IDTPARQUIVO']) {
                    case self::PAGAMENTO:
                        $this->importaRetornoPagamento($msgDisponivel);
                        break;
                    case self::RECEBIMENTO:
                        $this->importaRetornoCobranca($msgDisponivel);
                        break;
                    case self::OFX:
                        $this->importaExtratoOFX($msgDisponivel);
                        break;
                    case self::DDA:
                        $this->importaRetornoDDA($msgDisponivel);
                        break;
                }
            }
        }
    }
    
    /**
     * Method: importaRetornoPagamento()
     *
     * Description: Importa o arquivo de Retorno Pagamento recebido através do
     * WebService.
     *
     * @param $msgDisponivel
     */
    public function importaRetornoPagamento($msgDisponivel) {
        try {
            // Preparação de Dados - Carrega Retorno
            $row['CDBANCCNAB']  = substr($msgDisponivel['DSCONTTRCKIDACS'], 0, 3);
            $row['CDBANCO']     = substr($msgDisponivel['DSCONTTRCKIDACS'], 0, 3);
            $row['CDEMPRESA']   = self::$cdempresa;
            $row['NRORG']       = self::$nrorg;
            $row['CNABRETORNO'] = base64_encode($msgDisponivel['DSCONTTRCKIDACS']);
            $row['CDFILIAL']    = self::$cdfilial;
            $row['CDOPERADOR']  = self::$cdoperador;
            
            // Carrega Retorno Pagamento
            $dataSet = $this->retornoPagarService->carregaRetorno($row);
            
            // Preparação de Dados - Insere Retorno
            $boletos     = $dataSet[0];
            $ocorrencias = $dataSet[1];
            var_dump($ocorrencias);
            
            // Insere Retorno
            $retorno = $this->retornoPagarService->insereRetornoPagamento(
                $boletos, $ocorrencias, null, $row['CDEMPRESA'], $row['CDBANCO'],
                $row['NRORG'], $row['CDFILIAL'], $row['CDOPERADOR'], true
            );
            
            // Verifica Status da Importação
            $idstatimport = $retorno?self::CONCLUIDA:self::ERRO;
            
            // Atualiza Status da Importação da Faixa de Controle
            $this->importacaoBancariaFactory->updateIdstatimportCtrltrckidacs(
                $msgDisponivel['NRCTRLTRCKIDACS'], $msgDisponivel['DSTRACKINGID'], $idstatimport
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Method: importaRetornoCobranca()
     *
     * Description: Importa o arquivo de Retorno Cobrança recebido através do
     * WebService.
     *
     * @param $msgDisponivel
     */
    public function importaRetornoCobranca($msgDisponivel) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            // Preparação de Dados - Carrega Retorno
            $row['CDBANCCNAB']  = substr($msgDisponivel['DSCONTTRCKIDACS'], 0, 3);
            $row['CDBANCO']     = substr($msgDisponivel['DSCONTTRCKIDACS'], 0, 3);
            $row['CDEMPRESA']   = self::$cdempresa;
            $row['NRORG']       = self::$nrorg;
            $row['CNABRETORNO'] = base64_encode($msgDisponivel['DSCONTTRCKIDACS']);
            $row['CDFILIAL']    = self::$cdfilial;
            $row['CDOPERADOR']  = self::$cdoperador;
            
            // Carrega Retorno Cobrança
            $dataSet = $this->retornoCobrancaService->carregaRetorno($row);
            
            // Preparação de Dados - Insere Retorno
            $boletos     = $dataSet[0];
            $ocorrencias = $dataSet[1];
            var_dump($ocorrencias);
            
            // Insere Retorno
            $retorno = $this->retornoCobrancaService->insereRetornoCobranca(
                $boletos, $ocorrencias, '', $row['CDEMPRESA'], $row['CDBANCO'],
                $row['NRORG'], $row['CDFILIAL'], $row['CDOPERADOR'], true
            );
            
            // Verifica Status da Importação
            $idstatimport = $retorno?self::CONCLUIDA:self::ERRO;
            
            // Atualiza Status da Importação da Faixa de Controle
            $this->importacaoBancariaFactory->updateIdstatimportCtrltrckidacs(
                $msgDisponivel['NRCTRLTRCKIDACS'], $msgDisponivel['DSTRACKINGID'], $idstatimport
            );
        
            $this->entityManager->flush();
            $connection->commit();
        }catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Method: importaExtratoOFX()
     * 
     * Description: Essa função tem como finalidade verificar se o arquivo importado é do tipo TXT ou
     * OFX e chamar a função comrrespindende para lelos. Ambas as funções iram retornar
     * um array de duas posições, onde a primeira se trata dos dados do cabeçalho da
     * widget de imporatação, e a segunda posição as movimentações que deverão ser
     * salvas, e que iram aparecer no grid.
     *
     * @param $msgDisponivel
     */
    public function importaExtratoOFX($msgDisponivel) {
        
        $nrctrltrckidacs = $msgDisponivel['NRCTRLTRCKIDACS'];
        $dstrackingid    = $msgDisponivel['DSTRACKINGID'];
        $conteudo        = $msgDisponivel['DSCONTTRCKIDACS'];
        
        $conteudo = base64_encode($conteudo);
        $dataSets = $this->bco54200ExtratoBancario->readFileOfx($conteudo, self::$nrorg, true);
        
        $idstatimport = 'E';
        
        if(isset($dataSets['Error'])){
            var_dump($dataSets['Error']);
            $idstatimport = 'E';
        } else {
            
            $rows = $dataSets[1];
            
            $rows = $this->bco54200ExtratoBancario->removeMovimentosExistentesNoSistema($rows);
            $rowsArray = $this->bco54200ExtratoBancario->removeMovimentosDuplicadosNoArquivo($rows);
            $rows = $rowsArray[0];
            $movimentosDuplicados = $rowsArray[1];
            
            if (count($rows)){
                foreach($rows as $row) {
                    
                    //Faz as validações referentes a tabela de extrato, EXTRBANCO
                    $row = $this->bco54200ExtratoBancario->validaExtrato($row);
                    $this->entityManager->flush();

                    //Quando novo, insere a movimentação na tabela MOVREAL, quando edição, apenas updata o registro
                    if($row['__is_new']){
                        $this->bco54200ExtratoBancario->insertMovReal($row);
                    } else {
                        $this->bco54200ExtratoBancario->updateMovReal($row);
                    }
                    $this->entityManager->flush();

                    //Updata o extrato, recalculando o seu valor final.
                    $this->bco54200ExtratoBancario->recalculoExtrato($row);
                    $this->entityManager->flush();
                    
                    $idstatimport = 'C';
                }
            }else{
                var_dump('Os dados do arquivo ja foram importados no sistema.');
                $idstatimport = 'E';
            }
        }
        
        // Atualiza Status da Importação da Faixa de Controle
        $this->importacaoBancariaFactory->updateIdstatimportCtrltrckidacs(
            $nrctrltrckidacs, $dstrackingid, $idstatimport
        );
    }
    
    /**
     * Method: importaRetornoDDA()
     *
     * Description: Pega todos os dados do AccesStage que não foram concluidos e 
     * procuram o conteudo que seja de um arquivo OFX
     *
     * @param $msgDisponivel
     */
    public function importaRetornoDDA($msgDisponivel) {
        try{
            $dsconttrckidacs = $msgDisponivel['DSCONTTRCKIDACS'];
            
            // Preparação de Dados - Carrega Retorno
            $row['CDBANCCNAB']  = substr($dsconttrckidacs, 0, 3);
            $row['CDBANCO']     = substr($dsconttrckidacs, 0, 3);
            $row['CDEMPRESA']   = self::$cdempresa;
            $row['NRORG']       = self::$nrorg;
            $row['CNABRETORNO'] = base64_encode($dsconttrckidacs);
            $row['ARQUIVO']     = $dsconttrckidacs;
            $row['CDFILIAL']    = self::$cdfilial;
            $row['CDOPERADOR']  = self::$cdoperador;
            
            $dda = $this->pag30000RetornoDDAService->importaRetornoDDA($row, true);
            if(isset($dda[1]['Error'])){
                var_dump($dda[1]['Error']);
                $idstatimport = self::ERRO;
            }else $idstatimport = self::CONCLUIDA;
            
            // Atualiza Status da Importação da Faixa de Controle
            $this->importacaoBancariaFactory->updateIdstatimportCtrltrckidacs(
                $msgDisponivel['NRCTRLTRCKIDACS'], $msgDisponivel['DSTRACKINGID'], $idstatimport
            );
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    /**
     * Method: formatDate()
     *
     * Description: Formata data recebida pelo webservice (padrão AAAA-MM-DDTHH:MIN:SS.000-02:00) 
     * para o padrão utilizado pelo banco de dados (padrão DD/MM/AAAA HH:MIN:SS)
     *
     * @param $date
     */
    private function formatDate($date){
        return $date?date('d/m/Y H:i:s', strtotime($date)):date('d/m/Y H:i:s');
    }
    
    /**
     * Method: confirmaTrackId()
     *
     * Description: Confirmação de Mensagem para remoção do WebService
     *
     * @param $arquivo
     */
    /* private static function confirmaTrackId($arquivo){
        $arquivo['NMARQUIVORET'] = 'TESTE';
        
        //Formata data para o padrão AccesStage
        $DTRETIRADA = date("Y-m-dTG:i:s.000P");
        $DTRETIRADA = str_replace('BRT','T', $DTRETIRADA);
        
        $arquivo['DTRETIRADA'] = $DTRETIRADA;
        
        $arquivo["DSTRACKINGID"] = $arquivo["DSTRACKINGID"];
        
        $msgConfirmada = new \SoapClient(CONFIRMRETIRADA, array("keep_alive" => true));
        if(!$msgConfirmada){
            throw new \Exception("Não foi possível obter conexão para confirmar a retirada de arquivos.");
        }
        $msgConfirmada->__setSoapHeaders(self::$header);
        
        $parametro = ['trackingID' => $arquivo["DSTRACKINGID"], 'dataRetirada' => $arquivo['DTRETIRADA'], 'nmeArquivo' => $arquivo['NMARQUIVORET']];
        
        $response = $msgConfirmada->__soapCall('process', array($parametro));
        if($response->dscStatusRetirada != "Ok"){
            throw new \Exception("Não foi possível retirar o arquivo com o trakingId: ".$arquivo["DSTRACKINGID"]);
        }

        ImportacaoBancariaService::updateControleAcessWS($arquivo);

    }*/
    
}